#! /usr/bin/env python

import sys
from heapq import heappop, heappush


# direction enums
NORTH = (-1, 0)
SOUTH = (1, 0)
EAST  = (0, 1)
WEST  = (0, -1)


def vector_add(u, v):
    return tuple(map(lambda x, y: x + y, u, v))


class WaterMap(object):
    """Represent the basin-labeled elevation map with cells of 2-tuple:
    (height, label).
    """
    
    def __init__(self, map, height, width):
        self.height = height
        self.width = width
        self.map = {}
        for row in xrange(height):
            for col in xrange(width):
                self.map[(row,col)] = (map[row][col], None)

        self.current_label = 'a'
        self.label()

    def label(self):
        for row in xrange(self.height):
            for col in xrange(self.width):
                self._label(row, col)
    
    def _label(self, row, col):
        height, label = self.map[(row,col)]

        # base case: already labeled
        if label:
            return label

        lneighbor = self.get_lowest_neighbor(row, col)

        # base case: (row,col) is a sink
        if not lneighbor:
            assigned_label = self.current_label
            self.current_label = chr(ord(assigned_label) + 1)
            self.map[(row,col)] = (height, assigned_label)
            return assigned_label

        downstream_label = self._label(*lneighbor)
        self.map[(row,col)] = (height, downstream_label)
        return downstream_label

    def get_lowest_neighbor(self, row, col):
        cell = self.map[(row,col)]
        cell_height = cell[0]
        lowest_coord = None
        lowest_height = sys.maxint

        for neighbor in self.neighbors(row, col):
            ncell = self.map[neighbor]
            nheight = ncell[0]
            if nheight < cell_height and nheight < lowest_height:
                lowest_height = nheight
                lowest_coord = neighbor

        return lowest_coord

    def neighbors(self, row, col):
        coord = (row, col)

        if row > 0:
            yield vector_add(coord, NORTH)
        if col > 0:
            yield vector_add(coord, WEST)
        if col < self.width - 1:
            yield vector_add(coord, EAST)
        if row < self.height - 1:
            yield vector_add(coord, SOUTH)
    
    def __str__(self):
        row_list = []
        for row in xrange(self.height):
            col_list = []
            for col in xrange(self.width):
                label = self.map[(row,col)][1]
                if label:
                    col_list.append(label)
                else:
                    col_list.append('?')
            row_list.append(' '.join(col_list))
        
        return '\n'.join(row_list)


def main(argv):
    input = sys.stdin

    cases = int(input.readline())
    for case in xrange(cases):
        height, width = [int(num) for num in input.readline().split()]
        elevation_map = []
        for row in xrange(height):
            elevation_map.append([int(num) for num in input.readline().split()])
        
        water_map = WaterMap(elevation_map, height, width)
        print 'Case #%d:' % (case + 1)
        print water_map


if __name__ == '__main__':
    sys.exit(main(sys.argv))
